from django.shortcuts import render, HttpResponse, redirect

# Create your views here.

from django.http import JsonResponse
from django.contrib import auth
from blog import models
from blog.MyForms import UserForm
from blog import models
from django.db.models import Max,Avg,Count,Sum


def register(request):
    if request.method == 'POST':
        # print(request.POST)
        form = UserForm(request.POST)
        response = {"user": None, "msg": None}
        if form.is_valid():  # 如果验证通过,UserForm规范的字段
            # print(form.cleaned_data)
            response["user"] = form.cleaned_data.get("user")
            # 生成一条用户记录  create_user 会进行摘要
            user = form.cleaned_data.get("user")
            pwd = form.cleaned_data.get("pwd")
            email = form.cleaned_data.get("email")

            avatar_obj = request.FILES.get("avatar")  # Ajax传来的文件 这里必须是一个对象（文件和图片类似）
            extra = {}
            if avatar_obj:
                extra['avatar'] = avatar_obj
            # 没有文件，用默认的defalut.png，不用传值
            models.UserInfo.objects.create_user(username=user, password=pwd, email=email, **extra)
        else:
            # print(form.cleaned_data)
            # print(form.errors)
            response["msg"] = form.errors
        return JsonResponse(response)

    form = UserForm()
    return render(request, 'register.html', {'form': form})

def login(request):
    if request.method == 'POST':
        response = {'user': None, 'msg': None}
        user = request.POST.get('user')
        pwd = request.POST.get('pwd')
        valid_code = request.POST.get('valid_code')
        # 获取到后台保存的验证码
        random_char_last = request.session.get('random_char_last')
        if valid_code.upper() == random_char_last.upper():
            user = auth.authenticate(username=user, password=pwd)
            if user:  # 验证成功
                # 将user注册进去 request.user=当前登录对象
                auth.login(request, user)
                response['user'] = user.username
            else:
                response['msg'] = 'username or password error!'
        else:
            response['msg'] = 'valid code error!'
        return JsonResponse(response)  # 返回字典会自动序列化

    return render(request, 'login.html')

def logout(request):
    auth.logout(request)

    return redirect('/login/')

def get_validCode_img(requset):
    from blog.utils.validCode import get_valid_code_img
    data = get_valid_code_img(requset)

    return HttpResponse(data)

#系统首页
def index(request):
    #文章表中所有的数据
    article_list=models.Article.objects.all()

    return render(request, 'index.html',locals())

def site(request,username,**kwargs):
    #用户对象
    user = models.UserInfo.objects.filter(username=username).first()
    if not user:
        return render(request,'error_site_404.html')

    #个人站点blog对象
    blog_obj = user.blog
    #右侧 主体界面 文章
        #基于用户或当前站点对应的文章
    article_list=models.Article.objects.filter(user=user)
    if kwargs:
        path = kwargs.get('path')
        param = kwargs.get('param')
        if path == 'category':
            article_list = models.Article.objects.filter(user=user).filter(category__title=param)
        elif path == 'tag':
            article_list =models.Article.objects.filter(user=user).filter(tags__title=param)
        elif path =='month':
            y,m=param.split("-")
            article_list = models.Article.objects.filter(user=user).filter(create_time__year=y,create_time__month=m)

    #左侧随笔分类
        #查询当前站点的每一个分类名称以及对应的文章数
    category_list = models.Category.objects.filter(blog=blog_obj).values('pk').annotate(c=Count('article__title')).values_list('title','c')
    #左侧我的标签
        # 查询当前站点的每一个标签名称以及对应的文章数
    tag_list =models.Tag.objects.filter(blog=blog_obj).values('pk').annotate(c=Count('article__title')).values_list('title','c')

    #左侧随笔档案年月
        # 查询当前站点的每一个年月的名称以及对应的文章数  #mysql中用date_format(create_time,"%%Y%%m%%d")
    month_list = models.Article.objects.filter(user=user).extra(select={"create_time":"strftime('%%Y-%%m',create_time)"}).values("create_time").annotate(c=Count("nid")).values_list("create_time","c")


    return render(request,'site.html',locals())


def articles_detail(request,username,article_id):

    #用户对象
    user = models.UserInfo.objects.filter(username=username).first()
    if not user:
        return render(request,'error_site_404.html')

    blog =user.blog
    article_obj=models.Article.objects.filter(pk=article_id).first()
    # print(article_obj)
    comment_list = models.Comment.objects.filter(user=user,artcile_id=article_id).all()
    return render(request,'articles_detail.html',locals())

import json
from django.db.models import F

def digg(request):
    is_up=json.loads(request.POST.get("is_up")) #反序列化
    article_id =request.POST.get("article_id")
    user_id = request.user.pk

    obj = models.ArticleUpDown.objects.filter(user_id=user_id,artcile_id=article_id).first()
    response={"state":True} #判断有没有重复点击
    if not obj:#如果没有
        models.ArticleUpDown.objects.create(user_id=user_id,artcile_id=article_id,is_up=is_up)
        queryset=models.Article.objects.filter(pk=article_id)
        if is_up:
            queryset.update(up_count=F("up_count")+1)
        else:
            queryset.update(down_count=F("down_count")+1)

    else:
        response["state"]=False
        response["handled"]=obj.is_up  #判断是哪一个标签被点击
    return JsonResponse(response)

from django.db import transaction
def comment(request):
    """
    提交评论视图函数
    功能：
    1 保存评论
    2 创建事务 绑定
    3 发送邮件通知用户
    :param request:
    :return:
    """

    article_id = request.POST.get("article_id")
    pid = request.POST.get("pid")
    content = request.POST.get("content")
    user_id = request.user.pk

    article_obj = models.Article.objects.filter(pk=article_id).first()

    #事务操作 绑定在一起
    with transaction.atomic():
        comment_obj = models.Comment.objects.create(user_id=user_id, artcile_id=article_id, content=content,parent_comment_id=pid)
        models.Article.objects.filter(pk=article_id).update(comment_count=F("comment_count") + 1)

    response = {}
    response['create_time'] = comment_obj.create_time.strftime("%Y-%m-%d %X") #转换为字符串
    response["username"] = request.user.username
    response["content"] = content

    #发送邮件
    from django.core.mail import send_mail
    from cnblog import settings

    import threading
    t= threading.Thread(target=send_mail,args=(
        '您的文章%s新增了一条评论'%article_obj.title,
        content,
        settings.EMAIL_HOST_USER,
        ["zhangyang2@tydic.com"],
    ))

    # t.start()

    return JsonResponse(response)


def get_comment_tree(request):
    article_id = request.GET.get("article_id")

    response = list(models.Comment.objects.filter(artcile_id=article_id).order_by("pk").values("pk", "content", "parent_comment_id"))

    return JsonResponse(response, safe=False)

from django.contrib.auth.decorators import login_required
@login_required
def cn_backend(requset):
    '''
    后台管理首页
    :param requset:
    :return:
    '''
    article_list =models.Article.objects.filter(user=requset.user)
    # print(article_list)
    return render(requset, 'backend/backend.html', locals())


from bs4 import BeautifulSoup
@login_required
def add_article(request):
    '''
    后台新增随笔
    :param request:
    :return:
    '''
    if request.method == 'POST':
        title = request.POST.get('title')
        content = request.POST.get('content')
        print(title,content)
        #防xss脚本攻击，过滤script标签
        soup = BeautifulSoup(content,'html.parser')
        for tag in soup.find_all():
            if tag.name == 'script':
                tag.decompose()

        #构建摘要数据,获取标签字符串的文本前150个字符
        desc=soup.text[0:150]
        models.Article.objects.create(title=title,desc=desc,content=str(soup),user=request.user)
        return  redirect("/cn_backend/")

    return render(request,'backend/add_article.html')
@login_required
def delete_article(request,delete_article_id):
    models.Article.objects.filter(pk=delete_article_id).delete()
    return redirect("/cn_backend/")

@login_required
def edit_article(request,edit_article_id):

    if request.method == 'POST':
        title = request.POST.get('title')
        content = request.POST.get('content')
        print(title,content)
        #防xss脚本攻击，过滤script标签
        soup = BeautifulSoup(content,'html.parser')
        for tag in soup.find_all():
            if tag.name == 'script':
                tag.decompose()

        #构建摘要数据,获取标签字符串的文本前150个字符
        desc=soup.text[0:150]
        models.Article.objects.filter(pk=edit_article_id).update(title=title,desc=desc,content=str(soup),user=request.user)
        return  redirect("/cn_backend/")

    edit_obj=models.Article.objects.filter(pk=edit_article_id).first()
    return render(request,'backend/edit_article.html',locals())

import os
from cnblog import settings
def upload(request):
    """
    编辑器上传文件接受视图函数
    :param request:
    :return:
    """

    print(request.FILES)
    img_obj=request.FILES.get("upload_img")
    print(img_obj.name)

    path=os.path.join(settings.MEDIA_ROOT,"avatar",img_obj.name)

    with open(path,"wb") as f:

        for line in img_obj:
            f.write(line)

    response = {
        "error":0,
        "url":'media/avatar/%s'%(img_obj.name),

    }
    return HttpResponse(json.dumps(response))